/*
*******************************************************************************
*
* INC Name:  STDB_GEN
*
* Purpose:   This include file describes some fundamental data structures to
*	     be used throughout the STDB interface layer.  
*
* Modification History:
*
*   Date       OPR      Who                       Reason
* --------   -------    ---   -------------------------------------------------
* 11/21/91              lh    To make bind_type and context_type char strings
*                             fixed length (to reduce need for malloc) 
* 10/29/92    23669     sk    Add 3 fields: char_bind, endtab, numrow 
*                             to struct context_type
* 09/13/93              sk    in BIND_TYPE: delete ftn_len, add union
*                             in CONTEXT_TYPE: delete char_bind, syb_errno,
*                             add database_name, connection_established,
*                             status, alternate_status
* 10/07/93    25357     rjw   Get rid of IDM specific stuff
* 11/02/93              sk    add demand,(*error_handler)() to STDB_CONNECT_TYPE
* 11/09/93    25352     rjw   add return_len to BIND_TYPE for images/text
* 11/22/93    25351     rjw   add datetime_bind flag to context record
* 06/06/94              sk    Add define STRINGS_MATCH, OUTPUT_BOUND_VARIABLE,
*                             INPUT_PARAMETER_VARIABLE, 
*                             INPUT_NULL_PARAMETER_VARIABLE, 
*                             COLUMN_NOT_BOUND
* 06/10/94    26856     sk    Delete context[db].in_transaction field.
*                             Modify stdb_connection and context array size to
*                             STDB_MAX_CONNECT+1
* 07/18/94    25727     sk    add Structure STDB_SETUP_TYPE...
* 01/06/95    25826     sk    Add define RETURN_PARAM_VARIABLE 
* 02/17/95    28398	rjw   Add process_name to setup structure
* 07/06/95    29095	rjw   Add binary value to BIND_TYPE structure
* 02/26/96    30927     sk    Add tran_state on STDB_CONNECT array 
* 03/06/96              sk    change MAX_COMMAND_BUFFER size to handle text data
* 04/10/97    32535     sk    set password size to PASSWORD_MAX
* 04/17/97    32535     sk    include stdb.h (it's define PASSWORD_MAX)
* 05/08/97    32976	rjw   define username[USERNAME_MAX] 
* 06/23/97    34580     rjw   split bind array into separate arrays for
*                             params, binds, ret_params, special data
* 07/22/97              sk    remove first_param
* 05/06/98    36581     sk    change IMAGESIZE to string
* 05/13/98    36017     sk    add chval to param_val
* 06/22/98    37071     sk    add define MAX_STO_PARAM
*                             in CONTEXT_TYPE: add sto_param_count and
*                             *sto_param_value[MAX_STO_PARAM]; 
* 03/04/99              sk    move #define IMAGE_LEN to here
*******************************************************************************
*/

#ifndef STDB_GEN_INCLUDED
#define STDB_GEN_INCLUDED

#include "stdb.h"   /* where PASSWORD_MAX is defined */

/*************/
/* Constants */
/*************/

#define STDB_MAX_CONNECT   10     /* The maximum number of database process
                                     connections an image can have           */

#define MAX_STR            100    /* The maximum size of a string in STDB    */

#define MAX_VAR_ARRAY      250    /* Define the maxinum number of bind
                                     variables allowed in each query         */

#define MAX_STO_PARAM      255    /* Define the maxinum number of stored
                                     procedure parameters can have           */


#define MAX_COMMAND_BUFFER 1024*110 /* The maximum size of the query command 
				       buffer that is passed to DB machine   */

#define IMAGESIZE       500000    /* This is the maximum image size that
                                     STDB currently supports.  See image
                                     and text binding routines.              */

#define DEAD_NOT_OK          0    /* Deadlock is not valid, typically because
                                     you are in a transaction                */

#define DEAD_RETRY           1    /* Deadlock not in a transaction, so you can
                                     go ahead and retry to see if the deadlock
                                     condition may have been cleared         */

#define DEAD_MAXED           2    /* Deadlock not in a transaction, BUT, you
                                     have retried the query too many times,
                                     i.e.  more than this constant and it is
                                     time to get out of this query           */

#define STRINGS_MATCH        0    /* if two ASCII strings are equal          */

#define COLUMN_NOT_BOUND     0    /* column is not a retrieved column(output 
                                     bind variable, this flag is used in
                                     BIND_TYPE struct/col_num field          */

#define IMAGE_LEN      100        /* Maximum size of the image running query */

#define RPT_MAX_MSG   2000      /* maximum messages */
#define RPT_MSG_LENGTH  80 /* LENGTH of printing message segments */


/**************/
/* Structures */
/**************/

/*
 *
 * Structure
 * Name:     STDB_SETUP_TYPE
 *
 * Purpose:  Stores programmable fields that can be set via an (o)stdb_setup
 *           call.
 *
 */
typedef struct
{
  char username[USERNAME_MAX];        /* D = cuserid()                      */
  char password[PASSWORD_MAX];        /* D = ""                             */
  int  null_password;                 /* True or False, D = False           */
  char parameter_substitution_char;   /* D = @                              */
  int  deadlock_retry_max;            /* 0 - N , D = (DEAD_MAXED+1)= 3      */
  int  deadlock_pause_time;           /* > 1 seconds, D = 10                */
  char process_name[31];	      /* D = ""                             */
  int  textsize;                      /* the current value for DBTEXTSIZE   */ 
  
} STDB_SETUP_TYPE;


/*
 *
 * Structure
 * Name:     BIND_TYPE
 *
 * Purpose:  Stores bound db column vs. program variable information
 *
 */

typedef struct
{
  char name[MAX_STR];        /* Name of column being bound                 */
  
  int  col_num;              /* Column number of the bind variable in
				query                                      */
  
  int  type;                 /* datatype code of the binding variable,
				valid type are: SMALLBIND, FLT8BIND,
				REALBIND, BINARYBIND, STRINGBIND, INTBIND,
				CHARBIND, DATETIMEBIND, TEXTBIND           */
  
  int  len;                  /* length of returned string(bytes), must not
				be greater then the bound variable's size.
				if set to 0, the value retrieved from db will
				be copied to the bind variable, otherwise
				only copy the given len to the bind var    */
  
  int  *return_len;          /* returned length of variable - used for
				images and text                            */
/*
 * Bad news... Sybase calls this a long but it really isn't.  If you are
 * compiling 32 bit you have to call it a long, but if you are compiling 64
 * bit you have to call it an int.  Because of FreeTDS compatibility,
 * we now have to always define this as a long, and then dereference it
 * as a CS_INT in the code.
 */
  long long_return_len;      /* what ctlib actually uses, then we copy     */

  short indicator;           /* For ctlib, the indicator value */

  unsigned char *address;    /* output bind variable address               */

} BIND_TYPE;


/*
 *
 * Structure
 * Name:     PARAM_TYPE
 *
 * Purpose:  Stores input parameter variable information
 *
 */

typedef struct
{
  char name[MAX_STR];        /* Name of column being bound                 */
  
  int  type;                 /* datatype code of the parameter variable,
				valid types are: SMALLBIND, FLT8BIND, CHARBIND,
				REALBIND, BINARYBIND, STRINGBIND, INTBIND  */

  int  null_param;           /* TRUE if this parameter should be inserted as
                                "NULL"                                     */

  int  len;                  /* length of string or binary parameter       */

  union
  {
    short   sval;          /* short input parameter value                  */
    
    int     ival;          /* int input parameter value                    */
    
    float   fval;          /* float input parameter value                  */
    
    double  dval;          /* double input parameter value                 */

    char   chval;          /* single character parameter value             */
    
    char   *cval;          /* string input parameter address, since size
			      varies, use malloc to save string,
			      see module stdb_syb_interf.c function
			      stdb_syb_inbind_string                       */
    unsigned char 
    *bval;                 /* binary input parameter address, since size
		              varies, use malloc to save string,
		              see module stdb_syb_interf.c function
		              stdb_syb_inbind_binary                       */
    
  }  param_val;             /* parameter value                              */
  
} PARAM_TYPE;


/*
 * 
 * Structure
 * Name:       CONTEXT_TYPE
 * 
 * Purpose:    Basic process structure, contains db connection, bind, status
 *             information
 */

typedef struct
{
  int       demand;             /* demand connection flag, for sogs          */

  int       connection_number;  /* Old connection number for regression comp */

  int       tran_state;         /* transaction status                        */

  long      (*error_handler)(void *); /* this is address of either the STDB 
				   default error handler or a user supplied
				   error handler (via stdb_establish_error_
				   handler)                                  */

  void *syb_proc_ptr;            /* pointer to db process/connect structure  */

  void *syb_command_ptr;         /* pointer to the command/statement struct  */

  STDB_SETUP_TYPE setup;         /* Structure of setup values                */

  char *module_name;              /* Name of the module as passed to
				     stdb_query_setup                        */

  int  query_num;                 /* Number of query within above module as
				     passed to stdb_query_setup              */

  int  prev_qno;                  /* Previous query number within this context */

  int  rec_count;                 /* Number of records affected by the query;
				     gets incremented for each record
				     processed                               */

  char database_name[MAX_STR];    /* Name of database used for this context  */

  int  connection_established;    /* TRUE if database connection exists
				     for this context;  This gets reset to
				     FALSE when stdb_disconnect_dbms gets
				     called.                                 */

  int  status;                    /* Database dependent; initialize to 0
				     or "no error" at query_setup time;
				     set to database error code if an
				     error occurs                            */

  int  alternate_status;          /* User settable via a call to
				     stdb_set_alternate_status; User readable
				     via call to stdb_get_alternate_status   */

  int  get_state;                 /* query process state flag                */

  char command_buffer[MAX_COMMAND_BUFFER];
                                  /* This is the actual query text as passed
                                     to the database machine                 */

  int  closed;                    /* initial to True, set to False after
				     request query_setup, set to True after
				     invoke close_query                      */

  int  bind_count;                /* number of bind variables in the query   */

  int  param_count;               /* number of input parameters in the query */

  int  special_count;             /* number of special variables in the query*/

  int  ret_param_count;           /* number of return parameters in the query*/

  int  sto_param_count;           /* number of stored procedure parameters */

  int  stored_command;            /* set to TRUE if query is execute
				     stored procedure                        */
  int  endtab;                    /* end of fetch flag, in order to get
				     rec_cnt                                 */

  int    numrow;                  /* record count, to fix sybase DBROWS 
				     bug. initial to -1,                     */

  int    dead_count;              /* deadlock counter, for retry purpose     */

  int    deadlock_table_exists;   /* TRUE if stdb_deadlock table exists      */

  double tstart;                  /* process start time                      */

  double twall;                   /* process wall time                       */

  int    fortran_bind;            /* fortran bind string flag, when TRUE, this
				     means that we are binding FORTRAN data  */

  int    got_cancel;              /* set to TRUE if an stdb_cancel has been
				     called, FALSE otherwise. Note that this
                                     is reset to FALSE as soon as 
				     stdb_syb_check_interrupt_handler detects 
				     it so that the interrupt is not handled
				     repeatedly                              */

  int    cancel_query;            /* set to TRUE to indicate that the rest of
                                     the query results should be ignored by
                                     calling dbcanquery                      */


  BIND_TYPE bind[MAX_VAR_ARRAY];  /* bind variable information               */

  PARAM_TYPE param[MAX_VAR_ARRAY];
                                  /* input parameter information            */

  BIND_TYPE ret_param [MAX_VAR_ARRAY];  
                                  /* return parameter information for stored
                                     procedures                             */

  BIND_TYPE special[MAX_VAR_ARRAY];  
                                  /* special variables variable information -
                                     used for datetime, FORTRAN strings and
                                     text/image data which cannot be bound
                                     in the standard way                    */

  void *sto_param_value[MAX_STO_PARAM]; /* stored procedure parameters values */

  int morecmd;                   /* more commands for get_exec */
  int deadlock_table_loaded;     /* deadlock table loaded. */
  int allocated;                 /* context structure allocated */
  struct _stdb_driver *driver;   /* Drivers for this connection */

  int  index_err_msg;
  int  get_msg_index;
  char err_msg[RPT_MAX_MSG+1][RPT_MSG_LENGTH+1];

} CONTEXT_TYPE;

/* Database process structure array; contains STDB process information on the
 * current query. See above CONTEXT_TYPE struct definition for more details. 
 */
extern CONTEXT_TYPE       context[STDB_MAX_CONNECT+1];

typedef struct _stdb_driver {
    void (*init_library)(void);
    CONTEXT_TYPE *(*connect_dbms)(CONTEXT_TYPE *, char *, char *, const char *, const char *);
    int (*disconnect_dbms)(CONTEXT_TYPE *);
    int (*set_null)(CONTEXT_TYPE *, const int, const int, const void *);
    int (*query_setup)(CONTEXT_TYPE *, const char *, const int);
    int (*bind_binary)(CONTEXT_TYPE *, const char *, volatile const unsigned char *, const int);
    int (*inbind_binary)(CONTEXT_TYPE *, const char *, const unsigned char *, const int);
    int (*bind_int)(CONTEXT_TYPE *, const char *, volatile const int *);
    int (*int_ret_param)(CONTEXT_TYPE *, const char *, const int *);
    int (*inbind_int)(CONTEXT_TYPE *, const char *, const int);
    int (*inbind_single_char)(CONTEXT_TYPE *, const char *, const char);
    int (*bind_short)(CONTEXT_TYPE *, const char *, volatile const short *);
    int (*bind_single_char)(CONTEXT_TYPE *, const char *, volatile const char *);
    int (*single_char_ret_param)(CONTEXT_TYPE *, const char *, const char *);
    int (*short_ret_param)(CONTEXT_TYPE *, const char *, const short *);
    int (*inbind_short)(CONTEXT_TYPE *, const char *, const short);
    int (*bind_double)(CONTEXT_TYPE *, const char *, volatile const double *);
    int (*double_ret_param)(CONTEXT_TYPE *, const char *, const double *);
    int (*inbind_double)(CONTEXT_TYPE *, const char *, double);
    int (*bind_float)(CONTEXT_TYPE *, const char *, volatile const float *);
    int (*float_ret_param)(CONTEXT_TYPE *, const char *, const float *);
    int (*inbind_float)(CONTEXT_TYPE *, const char *, float);
    int (*string_ret_param)(CONTEXT_TYPE *, const char *, const char *, int);
    int (*bind_string)(CONTEXT_TYPE *, const char *, volatile const char *, int);
    int (*inbind_string)(CONTEXT_TYPE *, const char *, const char *);
    int (*inbind_null)(CONTEXT_TYPE *, const char *);
    int (*bind_text)(CONTEXT_TYPE *, const char *, volatile const void *, int *);
    int (*bind_image)(CONTEXT_TYPE *, const char *, volatile const void *, int *);
    int (*bind_datetime)(CONTEXT_TYPE *, const char *, volatile const STDB_DATEREC_TYPE *);
    int (*sql)(CONTEXT_TYPE *, const char *);
    int (*stored_command)(CONTEXT_TYPE *, const char *);
    int (*binary_param)(CONTEXT_TYPE *, const char *, const unsigned char *, const int);
    int (*single_char_param)(CONTEXT_TYPE *, const char *, const char);
    int (*int_param)(CONTEXT_TYPE *, const char *, const int);
    int (*short_param)(CONTEXT_TYPE *, const char *, const short);
    int (*double_param)(CONTEXT_TYPE *, const char *, const double);
    int (*real_param)(CONTEXT_TYPE *, const char *, const float);
    int (*string_param)(CONTEXT_TYPE *, const char *, const char *);
    int (*null_param)(CONTEXT_TYPE *, const char *);
    int (*get_exec)(CONTEXT_TYPE *);
    int (*close_query)(CONTEXT_TYPE *context);
    void (*cpu_time)(CONTEXT_TYPE *context);
    int (*record_count)(CONTEXT_TYPE *);
    int (*cur_cnt)(CONTEXT_TYPE *);
    int (*is_null)(CONTEXT_TYPE *, const void *);
    int (*set_textsize)(CONTEXT_TYPE *, const int);
    int (*cancel)(CONTEXT_TYPE *);
    void (*perf_rpt)(CONTEXT_TYPE *, const int, const double);
    void (*begin_transaction)(CONTEXT_TYPE *);
    void (*end_transaction)(CONTEXT_TYPE *);
    void (*abort_transaction)(CONTEXT_TYPE *);
    int (*in_transaction)(CONTEXT_TYPE *);
    int (*get_msg)(CONTEXT_TYPE *, char **);
} stdb_driver;

CONTEXT_TYPE *find_context(void *);

#endif 



